"""Utility module for freezing Python modules to a base binary."""

import cx_ShellUtils
import glob
import imp
import marshal
import modulefinder
import os
import struct
import sys

PYTHON_SHARED_LIB = None
if sys.platform == "win32":
    import win32api
    PYTHON_SHARED_LIB = win32api.GetModuleFileName(sys.dllhandle)

MAGIC = "cx_Freeze-v1"

# ensure that PyXML is handled properly if found
modulefinder.ReplacePackage("_xmlplus", "xml")

class ModuleFinder(modulefinder.ModuleFinder):
    """Subclass of modulefinder which handles a few situations that the
       base class does not handle very well."""

    def find_module(self, name, path, parent = None):
        """Find the module and return a reference to it."""
        if path is None and name in ("pythoncom", "pywintypes"):
            module = __import__(name)
            return None, module.__file__, (".dll", "rb", imp.C_EXTENSION)
        return modulefinder.ModuleFinder.find_module(self, name, path, parent)

    def load_module(self, fqname, fp, path, stuff):
        """Load the module and return a reference to it."""
        try:
            return modulefinder.ModuleFinder.load_module(self, fqname, fp,
                    path, stuff)
        except SyntaxError, value:
            raise "Module %s from file %s has %s." % (fqname, path, value)


def Freeze(targetFileName, moduleFinder, overridePath = True):
    """Freeze the modules found by the finder to the end of the file;
       return a list of additional copies that were made."""

    # force the use of the os module for extensions within packages
    moduleFinder.import_module("os", "os", None)

    # report the findings
    moduleFinder.report()

    # now write the modules to the end of the file
    # and the extensions to the directory the file is in
    tocLength = 0
    codeToWrite = []
    filesToCopy = []
    codeToWriteLength = 0
    outFile = file(targetFileName, "ab")
    baseDir = os.path.dirname(targetFileName)
    moduleNames = moduleFinder.modules.keys()
    moduleNames.sort()
    for name in moduleNames:
        module = moduleFinder.modules[name]
        if overridePath and module.__code__ is None \
                and module.__file__ is not None:
            dirName, fileName = os.path.split(module.__file__)
            baseFileName, ext = os.path.splitext(fileName)
            if baseFileName != name:
                if "." in name:
                    fileName = name + ext
                print "Module", name, "mangled to", fileName
                moduleCode = \
                        "import imp, os, sys\n" + \
                        "d, f = os.path.split(sys.executable)\n" + \
                        "f = os.path.join(d, '%s')\n" % fileName + \
                        "m = imp.load_dynamic(__name__, f)\n" + \
                        "sys.modules[__name__] = m"
                module.__code__ = compile(moduleCode, module.__file__, "exec")
            filesToCopy.append((module.__file__, fileName))
        if module.__code__ is not None:
            code = marshal.dumps(module.__code__)
            codeToWrite.append(code)
            codeLength = len(code)
            codeToWriteLength += codeLength
            nameLength = len(name)
            if nameLength % 4:
                nameLength += (4 - nameLength % 4)
            if module.__path__ is not None:
                codeLength = -codeLength
            tocEntry = struct.pack("i%dsi" % nameLength, nameLength, name,
                    codeLength)
            tocLength += len(tocEntry)
            outFile.write(tocEntry)
    for code in codeToWrite:
      outFile.write(code)
    headerData = struct.pack("iiii%ds" % len(MAGIC), codeToWriteLength,
        len(codeToWrite), tocLength, overridePath, MAGIC)
    outFile.write(headerData)
    outFile.close()
    print "File", targetFileName, "written."

    # don't copy any additional files if the path is being kept
    if not overridePath:
        return []

    # copy the Python shared library, if applicable
    if PYTHON_SHARED_LIB is not None:
        dir, name = os.path.split(PYTHON_SHARED_LIB)
        filesToCopy.append((PYTHON_SHARED_LIB, name))

    # copy the wxPython shared library, if applicable
    module = moduleFinder.modules.get("wxPython.wxc")
    if module is not None:
        if sys.platform == "win32":
            dir, name = os.path.split(module.__file__)
            dll, = glob.glob(os.path.join(dir, "wxmsw*.dll"))
            dir, name = os.path.split(dll)
            filesToCopy.append((dll, name))
        else:
            for line in os.popen("ldd %s" % module.__file__).readlines():
                line = line.strip()
                if line.startswith("libwx"):
                    libName, libLocation = line.split(" => ")
                    if libLocation == "not found":
                        raise "Library %s not found." % libName
                    libLocation = libLocation.split()[0]
                    dir, name = os.path.split(libLocation)
                    filesToCopy.append((libLocation, name))

    # return the list of additional files copied
    filesCopied = []
    for sourceName, targetName in filesToCopy:
        targetName = os.path.join(baseDir, targetName)
        print "Copying", sourceName
        cx_ShellUtils.Copy(sourceName, targetName, includeTimes = True)
        filesCopied.append(targetName)
    return filesCopied

