"""Extends readline capability on those platforms that support it."""

import getpass
import os
import sys

def AskBooleanQuestion(label):
    """Ask a boolean question repeatedly until one of Y or N is returned."""
    result = AskQuestion(label, ["Y", "N"])
    return result == "Y"

def AskQuestion(label, values):
    """Ask question repeatedly until one of the right answers is given. The
       first possible value is considered the default."""
    v_Label = label
    validValues = [s.upper() for s in values]
    defaultValue = validValues[0]
    stringValues = ['"%s"' % v for v in values]
    errorMessage = "Expecting one of %s\n" % " or ".join(stringValues)
    while True:
        value = ReadLine(label, defaultValue).upper()
        if value in validValues:
            break
        print errorMessage
        label = labelPrefix + label
    return value

def ReadLine(label, defaultValue = "", password = False):
    """Read a line of input from stdin and return it. If nothing is read, the
       default value is returned. Note that if stdin in not a TTY, the label
       will not be displayed."""
    if sys.stdin.isatty():
        if defaultValue:
            label += " [%s]" % defaultValue
        label += ": "
        if password:
            result = getpass.getpass(label)
        else:
            result = raw_input(label).strip()
    else:
        result = raw_input().strip()
    if not result:
        return defaultValue
    return result

# set up readline capability, if applicable
try:
    import readline
    readline.read_init_file()
except:
    pass

